/*
 * Lexical.java
 *
 * Created on 16  2002 ., 9:46
 */

package com.fn.spcave.compiler;


import java.io.File;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author  asso
 */
public class Lexical {
    
    private List code;
    private Project project;
    private Locale locale;
    private File currentFile;
    
    /** Creates a new instance of Lexical */
    public Lexical(Project prj, Locale locale, File file) {
        code = new ArrayList();
        project = prj;
        this.locale = locale;
        currentFile = file;
    }
    
    public void analyse(SourceReader sr) throws ScException {
        analyseBlock(sr, true);
    }
    
    public void analyseBlock(SourceReader sr, boolean toplevel) throws ScException {
        BeginInstruction begin = new BeginInstruction(this, sr.getName(), sr.getLineNumber(),
            code.size());
        code.add(begin);
        analyseBlock(sr, toplevel, begin);
    }
    
    private boolean isBlank(String s) {
        int len = s.length();
        for (int i = 0; i < len; i++)
            if (! Character.isWhitespace(s.charAt(i)))
                return false;
        return true;
    }
    
    public void analyseBlock(SourceReader sr, boolean toplevel, BeginInstruction begin) 
            throws ScException 
    {
        StringBuffer sb = new StringBuffer();
        String s;
        int ch = sr.getChar();
        while (ch != -1) {
            if (ch == '@') {
                int nch = sr.getChar();
                if (nch != '@') {
                    sr.ungetChar(nch);
                    s = sb.toString();
                    if (s.length() > 0) {
                        if (! isBlank(s))
                            code.add(new OutputInstruction(this, sr.getName(), 
                                    sr.getLineNumber(), code.size(), s));
                        sb.setLength(0);
                    }
                    String keyword = ReaderHelper.readWord(sr);
                    if (keyword.equals("end")) {
                        begin.setEndPosition(code.size());
                        code.add(new EndInstruction(this, sr.getName(), sr.getLineNumber(),
                                code.size(), begin.getPosition()));
                        return;
                    } else if (keyword.equals("begin")) {
                        BeginInstruction b = new BeginInstruction(this, sr.getName(), 
                                sr.getLineNumber(), code.size());
                        code.add(b);
                        analyseBlock(sr, false, b);
                        ch = sr.getChar();
                        continue;
                    } else if (keyword.equals("include")) {
                        if (currentFile == null)
                            throw new ScException("Inclusion is not allowed here.");
                        String fileName = readIncludeFilename(sr);
                        SourceReader isr = project.openIncludedFile(
                                currentFile.getParentFile(), locale, fileName);
                        analyseBlock(isr, true);
                        ch = sr.getChar();
                        continue;
                    } else {
                        if (KeywordsRegistry.isKeyword(keyword)) {
                            List args = null;
                            if (KeywordsRegistry.hasArgumets(keyword)) {
                                args = Utils.readFunctionArgs(sr);
                            }
                            KeywordInstruction ki = KeywordsRegistry.createKeywordInstruction(keyword, 
                                    this, sr.getName(), sr.getLineNumber(), code.size(), args);
                            code.add(ki);
                        } else {
                            List args = Utils.readFunctionArgs(sr);
                            FunctionHandler fh = FunctionsRegistry.getHandler(keyword);
                            if (fh != null) {
                                code.add(new FunctionInstruction(this, sr.getName(), sr.getLineNumber(), 
                                        code.size(), fh, args));
                            } else
                                throw new ScException(sr.getName() + ':' + 
                                    Integer.toString(sr.getLineNumber()) + ": Unknown keyword '" +
                                    keyword + "'");
                        }
                        ch = sr.getChar();
                        continue;
                    }
                }
            }
            sb.append((char)ch);
            ch = sr.getChar();
        }
        
        if (! toplevel)
            throw new ScException(sr.getName() + ':' + 
                Integer.toString(sr.getLineNumber()) + ": @end not found");
            
        s = sb.toString();
        if (s.length() > 0)
            code.add(new OutputInstruction(this, sr.getName(), sr.getLineNumber(), 
                    code.size(), s));
        begin.setEndPosition(code.size());
        code.add(new EndInstruction(this, sr.getName(), sr.getLineNumber(),
                code.size(), begin.getPosition()));
    }
    
    private String readIncludeFilename(SourceReader sr) throws ScException {
        String s = ReaderHelper.readToken(sr);
        if (! s.equals("("))
            throw new ScException(sr.getName() + ':' + Integer.toString(sr.getLineNumber()) +
                    ": '(' expected but '" + s + "' found");
        String name = ReaderHelper.readToken(sr);
        if (name.length() < 1)
            throw new ScException(sr.getName() + ':' + Integer.toString(sr.getLineNumber()) +
                    ": Invalide name for include");
        s = ReaderHelper.readToken(sr);
        if (! s.equals(")"))
            throw new ScException(sr.getName() + ':' + Integer.toString(sr.getLineNumber()) +
                    ": ')' expected but '" + s + "' found");
        ReaderHelper.skipSpaces(sr);
        return name;
    }
    
    public int getSize() {
        return code.size();
    }
    
    public Instruction getInstruction(int pos) throws ScException {
        if ((pos < 0) || (pos >= code.size()))
            throw new ScException("Attempt of excution of instruction at wrong address");
        return (Instruction)code.get(pos);
    }
}
