/*
 * SpidersCave.java
 *
 * Created on 6  2002 ., 8:32
 */

package com.fn.spcave.compiler;


import java.io.File;
import java.util.Iterator;


/**
 *
 * @author  asso
 */
public class SpidersCave {

    private Project project;
    private String projectFileName;
    
    /** Creates a new instance of SpidersCave */
    public SpidersCave(String args[]) {
        try {
            parseArgs(args);
            project = new Project(projectFileName);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    }

    private void showUsage() {
        System.out.println("USAGE:");
        System.out.println("    spcave <project_file_name>");
        System.exit(0);
    }
    
    private void parseArgs(String args[]) {
        if (args.length != 1) 
            showUsage();
        projectFileName = args[0];
    }
    
    private void processLocale(Locale locale) {
        if (locale != null)
            System.out.println("Processing locale: " + locale.getName());
        else
            System.out.println("Processing locale independed data");
        
        File dir;
        if ((locale == null) || locale.isDefault())
            dir = project.getOutputDir();
        else
            dir = new File(project.getOutputDir(), locale.getName());
        Compiler c = new Compiler(project.getSourceDir(), dir, locale, project);
        try {
            c.run();
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    }
    
    private void run() {
        File outputDir = project.getOutputDir();
        if (! outputDir.exists())
            outputDir.mkdirs();       
        processLocale(null);
        for (Iterator i = project.getLocalesIterator(); i.hasNext(); ) {
            Locale locale = (Locale)i.next();
            processLocale(locale);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        SpidersCave sp = new SpidersCave(args);
        sp.run();
    }
    
}
