/*
 * TextFile.java
 *
 * Created on 15  2002 ., 17:36
 */

package com.fn.spcave.compiler;


import java.io.File;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author  asso
 */
public class TextFile {
    
    private Project project;
    private Locale locale;
    private Map messages;
    private Map parsedMessages;
    
    /** Creates a new instance of TextFile */
    public TextFile(Project project, File currentDir, String fileName, 
                Locale locale) throws ScException 
    {
        this.project = project;
        this.locale = locale;
        SourceReader sr = project.openIncludedFile(currentDir, locale, fileName);
        messages = new HashMap();
        parsedMessages = new HashMap();
        loadFile(sr);
    }

    private void loadFile(SourceReader sr) throws ScException {
        String name = ReaderHelper.readToken(sr);
        while (name.length() > 0) {
            String word = ReaderHelper.readToken(sr);
            if (! word.equals("="))
                throw new ScException("Invalid symbol in message file: '=' expected but '"
                        + word + "' found.");
            String text = ReaderHelper.readTextBlock(sr);
            messages.put(name, text);
            name = ReaderHelper.readToken(sr);
        }
    }
    
    public void processMessage(Interpreter interp, String name) throws ScException {
        Lexical code;
        code = (Lexical)parsedMessages.get(name);
        if (code == null) {
            String s = (String)messages.get(name);
            if (s == null)
                return;
            SourceReader sr = new ChainedSourceReader("messages:" + name, 0, new StringReader(s));
            code = new Lexical(project, locale, null);
            code.analyse(sr);
            parsedMessages.put(name, code);
            messages.remove(name);
        }
        interp.processBlock(code, 0);
    }
}
