/*
 * NewsKeyword.java
 *
 * Created on 22  2002 ., 10:55
 */

package com.fn.station;


import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import com.fn.spcave.compiler.*;


/**
 *
 * @author  asso
 */
public class NewsKeyword extends KeywordInstruction {
    
    /** Creates a new instance of NewsKeyword */
    public NewsKeyword(Lexical lexical, String source, int line, int position, 
            List arguments) throws ScException {
        super(lexical, source, line, position, arguments);
        if (arguments.size() != 3)
            throw new ScException(getErrorPos() + "news takes exacly 3 arguments");
    }

    
    public int processKeyword(Interpreter interp) throws ScException {
        String newsFileName = ((Expression)getArguments().get(0)).asString(interp);
        int maxNewsCount = (int)((Expression)getArguments().get(1)).asDouble(interp);
        int year = (int)((Expression)getArguments().get(2)).asDouble(interp);
        int pos = getPosition() + 1;
        Instruction instr = getLexical().getInstruction(pos);
        if (instr.getCode() != BeginInstruction.BEGIN_CODE)
            throw new ScException(getErrorPos() + "@news must followed by @begin - @end block");
        BeginInstruction bi = (BeginInstruction)instr;
        
        File f = interp.getProject().getIntlFile(interp.getCurrentDir(), 
                    interp.getLocale(), newsFileName);

        NewsFile newsFile = new NewsFile(f);
        int printedNewsCount = 0;
        for (Iterator i = newsFile.getIterator(); i.hasNext(); ) {
            NewsFile.NewsEntry ne = (NewsFile.NewsEntry)i.next();
            if ((year <= 0) || (year == ne.year)) {
                interp.setVariable("news_title", ne.title);
                interp.setVariable("news_descr", ne.descr);
                interp.setVariable("news_body", ne.body);
                interp.setVariable("news_day", Integer.toString(ne.day));
                interp.setVariable("news_month", Integer.toString(ne.month));
                interp.setVariable("news_year", Integer.toString(ne.year));
                int ip = interp.processBlock(getLexical(), bi.getPosition());
                if (bi.isBreaked())
                    return ip;
                printedNewsCount++;
            }
            if ((maxNewsCount >= 0) && (printedNewsCount >= maxNewsCount))
                break;
        }
        return bi.getEndPosition() + 1;
    }
}
